@@ -23,6 +23,8 @@ import (
 	"fmt"
 	"net/http"
 	"net/url"
+	"strconv"
+	"time"
 
 	"github.com/cyberphone/json-canonicalization/go/src/webpki.org/jsoncanonicalizer"
 	"github.com/go-openapi/runtime"
@@ -58,15 +60,15 @@ const (
 func signEntry(ctx context.Context, signer signature.Signer, entry models.LogEntryAnon) ([]byte, error) {
 	payload, err := entry.MarshalBinary()
 	if err != nil {
-		return nil, fmt.Errorf("marshalling error: %v", err)
+		return nil, fmt.Errorf("marshalling error: %w", err)
 	}
 	canonicalized, err := jsoncanonicalizer.Transform(payload)
 	if err != nil {
-		return nil, fmt.Errorf("canonicalizing error: %v", err)
+		return nil, fmt.Errorf("canonicalizing error: %w", err)
 	}
 	signature, err := signer.SignMessage(bytes.NewReader(canonicalized), options.WithContext(ctx))
 	if err != nil {
-		return nil, fmt.Errorf("signing error: %v", err)
+		return nil, fmt.Errorf("signing error: %w", err)
 	}
 	return signature, nil
 }
@@ -185,7 +187,8 @@ func createLogEntry(params entries.CreateLogEntryParams) (models.LogEntry, middl
 	}
 	leaf, err := types.CanonicalizeEntry(ctx, entry)
 	if err != nil {
-		if _, ok := (err).(types.ValidationError); ok {
+		var validationErr *types.InputValidationError
+		if errors.As(err, &validationErr) {
 			return nil, handleRekorAPIError(params, http.StatusBadRequest, err, fmt.Sprintf(validationError, err))
 		}
 		return nil, handleRekorAPIError(params, http.StatusInternalServerError, err, failedToGenerateCanonicalEntry)
@@ -246,6 +249,14 @@ func createLogEntry(params entries.CreateLogEntryParams) (models.LogEntry, middl
 
 	if indexStorageClient != nil {
 		go func() {
+			start := time.Now()
+			var err error
+			defer func() {
+				labels := map[string]string{
+					"success": strconv.FormatBool(err == nil),
+				}
+				metricIndexStorageLatency.With(labels).Observe(float64(time.Since(start)))
+			}()
 			keys, err := entry.IndexKeys()
 			if err != nil {
 				log.ContextLogger(ctx).Errorf("getting entry index keys: %v", err)
@@ -277,12 +288,12 @@ func createLogEntry(params entries.CreateLogEntryParams) (models.LogEntry, middl
 
 	signature, err := signEntry(ctx, api.signer, logEntryAnon)
 	if err != nil {
-		return nil, handleRekorAPIError(params, http.StatusInternalServerError, fmt.Errorf("signing entry error: %v", err), signingError)
+		return nil, handleRekorAPIError(params, http.StatusInternalServerError, fmt.Errorf("signing entry error: %w", err), signingError)
 	}
 
 	root := &ttypes.LogRootV1{}
 	if err := root.UnmarshalBinary(resp.GetLeafAndProofResult.SignedLogRoot.LogRoot); err != nil {
-		return nil, handleRekorAPIError(params, http.StatusInternalServerError, fmt.Errorf("error unmarshalling log root: %v", err), sthGenerateError)
+		return nil, handleRekorAPIError(params, http.StatusInternalServerError, fmt.Errorf("error unmarshalling log root: %w", err), sthGenerateError)
 	}
 	hashes := []string{}
 	for _, hash := range resp.GetLeafAndProofResult.Proof.Hashes {
@@ -405,8 +416,9 @@ func GetLogEntryByUUIDHandler(params entries.GetLogEntryByUUIDParams) middleware
 		if errors.Is(err, ErrNotFound) {
 			return handleRekorAPIError(params, http.StatusNotFound, err, "")
 		}
-		if _, ok := (err).(types.ValidationError); ok {
-			return handleRekorAPIError(params, http.StatusBadRequest, err, fmt.Sprintf("incorrectly formatted uuid %s", params.EntryUUID), params.EntryUUID)
+		var validationErr *types.InputValidationError
+		if errors.As(err, &validationErr) {
+			return handleRekorAPIError(params, http.StatusBadRequest, err, fmt.Sprintf("validation error: %v", err))
 		}
 		return handleRekorAPIError(params, http.StatusInternalServerError, err, trillianCommunicationError)
 	}
@@ -592,7 +604,7 @@ func retrieveUUIDFromTree(ctx context.Context, uuid string, tid int64) (models.L
 
 	hashValue, err := hex.DecodeString(uuid)
 	if err != nil {
-		return models.LogEntry{}, types.ValidationError(err)
+		return models.LogEntry{}, &types.InputValidationError{Err: fmt.Errorf("parsing UUID: %w", err)}
 	}
 
 	tc := trillianclient.NewTrillianClient(ctx, api.logClient, tid)
